local TMA_Menu = _G["TMA_Menu"]
if not TMA_Menu then return end

-- Sample panel data and option controls
local function AddMenuContents()
    -- Sample panel data
    local panelData = {
        panelName = "TealkeMultiAddonsPanel",
        displayName = "Tealke Multi Addons",
        version = "1.0",
        author = "Tealke",
    }

    -- Register the panel
    TMA_Menu:RegisterAddonPanel("TealkeMultiAddonsPanel", panelData)

    -- Sample option controls
    local optionsTable = {
        {
            type = "Description",
            text = "This is a description for the Tealke Multi Addons panel.",
            tooltip = "This is a tooltip for the description."
        },
        {
            type = "Dropdown",
            name = "Select an Option",
            choices = {"Option 1", "Option 2", "Option 3"},
            tooltip = "Select an option from the dropdown.",
            setFunc = function(value) d("Dropdown selected: " .. value) end,
        },
        {
            type = "Slider",
            name = "Adjust the Slider",
            min = 0,
            max = 100,
            step = 1,
            getFunc = function() return 50 end,
            setFunc = function(value) d("Slider value: " .. value) end,
            tooltip = "Adjust the slider between 0 and 100.",
        },
        {
            type = "Button",
            name = "Click Me",
            func = function() d("Button clicked") end,
            tooltip = "Click this button.",
        },
        {
            type = "Checkbox",
            name = "Enable Feature",
            getFunc = function() return true end,
            setFunc = function(value) d("Checkbox state: " .. tostring(value)) end,
            tooltip = "Check or uncheck this box.",
        },
        {
            type = "ColorPicker",
            name = "Pick a Color",
            getFunc = function() return 1, 0, 0, 1 end,
            setFunc = function(r, g, b, a) d("Color selected: ", r, g, b, a) end,
            tooltip = "Pick a color.",
        },
        {
            type = "EditBox",
            name = "Enter Text",
            getFunc = function() return "Sample text" end,
            setFunc = function(text) d("EditBox text: " .. text) end,
            tooltip = "Enter some text.",
        },
        {
            type = "Submenu",
            name = "Submenu",
            controls = {
                {
                    type = "Checkbox",
                    name = "Submenu Checkbox",
                    getFunc = function() return false end,
                    setFunc = function(value) d("Submenu checkbox state: " .. tostring(value)) end,
                    tooltip = "This is a checkbox in a submenu.",
                },
            },
        },
    }

    -- Create the settings window and register option controls
    TMA_Menu:CreateSettingsWindow("TealkeMultiAddonsPanel_SettingsWindow")
    TMA_Menu:RegisterOptionControls("TealkeMultiAddonsPanel", optionsTable)
end

-- Register the settings and add menu contents on addon load
EVENT_MANAGER:RegisterForEvent("TMA_Menu_Load", EVENT_ADD_ON_LOADED, function(event, addonName)
    if addonName == "TealkeMultiAddons" then
        EVENT_MANAGER:UnregisterForEvent("TMA_Menu_Load", EVENT_ADD_ON_LOADED) --unregister asfter your addon was found!
        TMA_Menu.RegisterSettings()
        TMA_Menu:HideSettingsWindow()
        AddMenuContents()
    end
end)